@@ -3,7 +3,7 @@
  * @brief NVME function declarations
  * @details This header file declares some NVME related functions
  * @copyright @verbatim
-Copyright © 2011 - 2023 Petros Koutoupis
+Copyright © 2011 - 2025 Petros Koutoupis
 
 All rights reserved.
 
@@ -26,8 +26,8 @@ SPDX-License-Identifier: GPL-2.0-or-later
 @endverbatim
  * @author Petros Koutoupis \<petros\@petroskoutoupis.com\>
  * @author Matteo Tenca \<matteo.tenca\@gmail.com\>
- * @version 9.1.0
- * @date 23 April 2023
+ * @version 9.2.0
+ * @date 15 March 2025
  */
 
 #ifndef NVMET_H
@@ -43,6 +43,7 @@ struct NVMET_PROFILE *nvmet_scan_subsystem(char *return_message, int *rc);
 struct NVMET_PORTS *nvmet_scan_ports(char *return_message, int *rc);
 struct NVMET_PORTS *nvmet_scan_all_ports(char *return_message, int *rc);
 char *nvmet_interface_ip_get(char *interface, char *return_message);
+bool check_nvmet_subsystem(char *return_message);
 #ifndef SERVER
 int nvmet_view_exports(bool json_flag, char *error_message);
 int nvmet_view_ports(bool json_flag, char *error_message);
