@@ -3,7 +3,7 @@
  * @brief Utility functions declarations
  * @details This header file defines some utility functions
  * @copyright @verbatim
-Copyright © 2011 - 2023 Petros Koutoupis
+Copyright © 2011 - 2025 Petros Koutoupis
 
 All rights reserved.
 
@@ -26,8 +26,8 @@ SPDX-License-Identifier: GPL-2.0-or-later
 @endverbatim
  * @author Petros Koutoupis \<petros\@petroskoutoupis.com\>
  * @author Matteo Tenca \<matteo.tenca\@gmail.com\>
- * @version 9.1.0
- * @date 23 April 2023
+ * @version 9.2.0
+ * @date 15 March 2025
  */
 #ifndef UTILS_H
 #define UTILS_H
@@ -43,5 +43,5 @@ struct dirent **clean_scandir(struct dirent **scanlist, int num);
 int check_loaded_modules(void);
 void print_message(int ret_value, char *message, bool json_flag);
 char *verbose_msg(char *dest, char *msg);
-
+unsigned long long int validate_size(const char *re, char *subject, char *error_message);
 #endif //UTILS_H
